import json
import os
import shutil
import unittest
from satstac import Thing, STACError


testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    path = os.path.join(testpath, 'test-thing')
    fname = os.path.join(testpath, 'catalog/catalog.json')

    @classmethod
    def tearDownClass(cls):
        """ Remove test files """
        if os.path.exists(cls.path):
            shutil.rmtree(cls.path)

    def get_thing(self):
        """ Configure testing class """
        return Thing.open(self.fname)

    def test_init_error(self):
        with self.assertRaises(STACError):
            Thing({})

    def test_init(self):
        thing1 = self.get_thing()
        assert(thing1.id == 'stac')
        assert(len(thing1.links()) == 3)
        assert(len(thing1.links('self')) == 1)
        data = thing1._data
        del data['links']
        thing2 = Thing(data)
        assert(thing2.links() == [])
        with self.assertRaises(STACError):
            thing2.save()
        assert(thing2.id == str(thing2))

    def test_open(self):
        thing1 = self.get_thing()
        thing2 = Thing.open(self.fname)
        assert(thing1.id == thing2.id)
        assert(
            os.path.basename(thing1.links()[0]) 
            == os.path.basename(thing2.links()[0])
        )
        assert(thing2.path == os.path.dirname(self.fname))

    def test_open_missing(self):
        with self.assertRaises(STACError):
            thing = Thing.open('nosuchfile.json')        

    def test_open_remote(self):
        thing = Thing.open('https://landsat-stac.s3.amazonaws.com/catalog.json')
        assert(thing.id == 'landsat-stac')
        assert(len(thing._data['links']) == 3)

    def test_open_missing_remote(self):
        with self.assertRaises(STACError):
            thing = Thing.open('https://landsat-stac.s3.amazonaws.com/nosuchcatalog.json')

    def test_thing(self):
        thing = self.get_thing()
        assert('id' in thing._data.keys())
        assert('links' in thing._data.keys())
        del thing._data['links']
        assert('links' not in thing._data.keys())
        assert(thing.links() == [])

    def test_get_links(self):
        thing = Thing.open('https://landsat-stac.s3.amazonaws.com/catalog.json')
        things = [Thing.open(t) for t in thing.links('child')]
        assert('landsat-8-l1' in [t.id for t in things])

    def test_add_link(self):
        thing = self.get_thing()
        thing.clean_hierarchy()
        thing.filename = None
        thing.add_link('testlink', 'bobloblaw', type='text/plain', title='BobLoblaw')
        assert(len(thing.links()) == 1)
        # try adding it again, should not add it if rel and href the same
        thing.add_link('testlink', 'bobloblaw', type='text/plain', title='BobLoblaw')
        assert(len(thing.links()) == 1)
        assert(len(thing.links('testlink')) == 1)
        assert(thing.links('testlink')[0] == 'bobloblaw')

    def test_get_root(self):
        thing = self.get_thing()
        root = thing.root()
        assert(root.filename == os.path.join(testpath, 'catalog/catalog.json'))
        thing.add_link('root', 'root', title='root2')
        with self.assertRaises(STACError):
            thing.root()
        thing.clean_hierarchy()
        root = thing.root()
        assert(root == thing)

    def test_get_parent(self):
        thing = Thing.open(os.path.join(testpath, 'catalog/eo/catalog.json'))
        parent = thing.parent()
        assert(parent.filename == os.path.join(testpath, 'catalog/catalog.json'))
        thing.add_link('parent', 'catalog.json', title='parent2')
        with self.assertRaises(STACError):
            thing.parent()
        thing.clean_hierarchy()
        parent = thing.parent()
        assert(parent is None)

    def test_clean_hierarchy(self):
        thing = self.get_thing()
        thing.add_link('testlink', 'bobloblaw')
        assert(len(thing.links()) == 4)
        thing.clean_hierarchy()
        assert(len(thing.links()) == 1)

    def test_getitem(self):
        thing = self.get_thing()
        assert(thing['some_property'] is None)
    
    def test_save(self):
        thing = Thing.open(self.fname)
        thing.save()
        fout = os.path.join(self.path, 'test-save.json')
        thing.save(fout)
        assert(os.path.exists(fout))

    def test_save_remote_with_signed_url(self):
        thing = Thing.open(self.fname)
        thing.save('https://landsat-stac.s3.amazonaws.com/test/thing.json')

    def test_save_remote_with_bad_signed_url(self):
        envs = dict(os.environ)
        thing = Thing.open(self.fname)
        os.environ['AWS_BUCKET_REGION'] = 'us-east-1'
        with self.assertRaises(STACError):
            thing.save('https://landsat-stac.s3.amazonaws.com/test/thing.json')
        os.environ.clear()
        os.environ.update(envs)



import logging
import functools
import os

from datetime import datetime

from .catalog import Catalog
from satstac import STACError, utils

logger = logging.getLogger(__name__)


class Collection(Catalog):

    '''
    def __init__(self, *args, **kwargs):
        """ Initialize a scene object """
        super(Collection, self).__init__(*args, **kwargs)
        # determine common_name to asset mapping
        # it will map if an asset contains only a single band
    '''

    @property
    def title(self):
        return self._data.get('title', '')

    @property
    def keywords(self):
        return self._data.get('keywords', [])

    @property
    def version(self):
        return self._data.get('version', '')

    @property
    def license(self):
        return self._data.get('license')

    @property
    def providers(self):
        return self._data.get('providers', [])

    @property
    def extent(self):
        return self._data.get('extent')

    @property
    def properties(self):
        """ Get dictionary of properties """
        return self._data.get('properties', {})

    @functools.lru_cache()
    def parent_catalog(self, path):
        """ Given path to a new Item find parent catalog """
        cat = self
        dirs = utils.splitall(path)
        var_names = [v.strip('$').strip('{}') for v in dirs]
        for i, d in enumerate(dirs):
            fname = os.path.join(os.path.join(cat.path, d), 'catalog.json')
            # open existing sub-catalog or create new one
            try:
                subcat = Catalog.open(fname)
            except STACError as err:
                # create a new sub-catalog
                subcat = self.create(id=d, description='%s catalog' % var_names[i])
                subcat.save(filename=fname)
                # add the sub-catalog to this catalog
                cat.add_catalog(subcat)
            cat = subcat
        return cat.filename

    def add_item(self, item, path='', filename='${id}'):
        """ Add an item to this collection """
        start = datetime.now()
        if self.filename is None:
            raise STACError('Save catalog before adding items')
        item_link = item.get_filename(path, filename)
        item_fname = os.path.join(self.path, item_link)
        item_path = os.path.dirname(item_fname)
        root_link = self.links('root')[0]
        root_path = os.path.dirname(root_link)

        parent = Catalog.open(self.parent_catalog(item.substitute(path)))
        
        # create link to item
        parent.add_link('item', os.path.relpath(item_fname, parent.path))
        parent.save()

        # create links from item
        item.clean_hierarchy()
        item.add_link('root', os.path.relpath(root_link, item_path))
        item.add_link('parent', os.path.relpath(parent.filename, item_path))
        # this assumes the item has been added to a Collection, not a Catalog
        item.add_link('collection', os.path.relpath(self.filename, item_path))

        # save item
        item.save(filename=item_fname)
        logger.debug('Added %s in %s seconds' % (item.filename, datetime.now()-start))
        return self



import base64
import calendar
import collections
import datetime
import hashlib
import hmac
import logging
import os
import requests
import sys
import time

logger = logging.getLogger(__name__)


# from https://gist.github.com/angstwad/bf22d1822c38a92ec0a9#gistcomment-2622319
def dict_merge(dct, merge_dct, add_keys=True):
    """ Recursive dict merge. Inspired by :meth:``dict.update()``, instead of
    updating only top-level keys, dict_merge recurses down into dicts nested
    to an arbitrary depth, updating keys. The ``merge_dct`` is merged into
    ``dct``.

    This version will return a copy of the dictionary and leave the original
    arguments untouched.

    The optional argument ``add_keys``, determines whether keys which are
    present in ``merge_dict`` but not ``dct`` should be included in the
    new dict.

    Args:
        dct (dict) onto which the merge is executed
        merge_dct (dict): dct merged into dct
        add_keys (bool): whether to add new keys

    Returns:
        dict: updated dict
    """
    dct = dct.copy()
    if not add_keys:
        merge_dct = {
            k: merge_dct[k]
            for k in set(dct).intersection(set(merge_dct))
        }

    for k, v in merge_dct.items():
        if (k in dct and isinstance(dct[k], dict)
                and isinstance(merge_dct[k], collections.Mapping)):
            dct[k] = dict_merge(dct[k], merge_dct[k], add_keys=add_keys)
        else:
            dct[k] = merge_dct[k]

    return dct


def download_file(url, filename=None, requestor_pays=False):
    """ Download a file as filename """
    filename = os.path.basename(url) if filename is None else filename
    logger.info('Downloading %s as %s' % (url, filename))
    headers = {}
    # check if on s3, if so try to sign it
    if 's3.amazonaws.com' in url:
        signed_url, signed_headers = get_s3_signed_url(url, requestor_pays=requestor_pays)
        resp = requests.get(signed_url, headers=signed_headers, stream=True)
        if resp.status_code != 200:
            resp = requests.get(url, headers=headers, stream=True)
    else:
        resp = requests.get(url, headers=headers, stream=True)
    if resp.status_code != 200:
        raise Exception("Unable to download file %s: %s" % (url, resp.text))
    with open(filename, 'wb') as f:
        for chunk in resp.iter_content(chunk_size=1024):
            if chunk:  # filter out keep-alive new chunks
                f.write(chunk)
    return filename


def mkdirp(path):
    """ Recursively make directory """
    if not os.path.isdir(path) and path != '':
        os.makedirs(path)
    return path


# from https://www.oreilly.com/library/view/python-cookbook/0596001673/ch04s16.html
def splitall(path):
    allparts = []
    while 1:
        parts = os.path.split(path)
        if parts[0] == path:  # sentinel for absolute paths
            allparts.insert(0, parts[0])
            break
        elif parts[1] == path: # sentinel for relative paths
            allparts.insert(0, parts[1])
            break
        else:
            path = parts[0]
            allparts.insert(0, parts[1])
    return allparts


def get_s3_signed_url(url, rtype='GET', public=False, requestor_pays=False, content_type=None):
    access_key = os.environ.get('AWS_BUCKET_ACCESS_KEY_ID', os.environ.get('AWS_ACCESS_KEY_ID'))
    secret_key = os.environ.get('AWS_BUCKET_SECRET_ACCESS_KEY', os.environ.get('AWS_SECRET_ACCESS_KEY'))
    region = os.environ.get('AWS_BUCKET_REGION', os.environ.get('AWS_REGION', 'eu-central-1'))
    if access_key is None or secret_key is None:
        # if credentials not provided, just try to download without signed URL
        logger.debug('Not using signed URL for %s' % url)
        return url, None

    parts = url.replace('https://', '').split('/')
    bucket = parts[0].replace('.s3.amazonaws.com', '')
    key = '/'.join(parts[1:])

    service = 's3'
    host = '%s.s3.amazonaws.com' % (bucket)
    request_parameters = ''

    # Key derivation functions. See:
    # http://docs.aws.amazon.com/general/latest/gr/signature-v4-examples.html#signature-v4-examples-python
    def sign(key, msg):
        return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

    def getSignatureKey(key, dateStamp, regionName, serviceName):
        kDate = sign(('AWS4' + key).encode('utf-8'), dateStamp)
        kRegion = sign(kDate, regionName)
        kService = sign(kRegion, serviceName)
        kSigning = sign(kService, 'aws4_request')
        return kSigning

    # Create a date for headers and the credential string
    t = datetime.datetime.utcnow()
    amzdate = t.strftime('%Y%m%dT%H%M%SZ')
    datestamp = t.strftime('%Y%m%d') # Date w/o time, used in credential scope

    # create signed request and headers
    canonical_uri = '/' + key
    canonical_querystring = request_parameters

    payload_hash = 'UNSIGNED-PAYLOAD'
    headers = {
        'host': host,
        'x-amz-content-sha256': payload_hash,
        'x-amz-date': amzdate
    }
    if requestor_pays:
        headers['x-amz-request-payer'] = 'requester'
    if public:
        headers['x-amz-acl'] = 'public-read'
    if os.environ.get('AWS_SESSION_TOKEN') and 'AWS_BUCKET_ACCESS_KEY_ID' not in os.environ:
        headers['x-amz-security-token'] = os.environ.get('AWS_SESSION_TOKEN')
    canonical_headers = '\n'.join('%s:%s' % (key, headers[key]) for key in sorted(headers)) + '\n'
    signed_headers = ';'.join(sorted(headers.keys()))

    canonical_request = '%s\n%s\n%s\n%s\n%s\n%s' % (
        rtype, canonical_uri, canonical_querystring, canonical_headers, signed_headers, payload_hash
    )
    algorithm = 'AWS4-HMAC-SHA256'
    credential_scope = datestamp + '/' + region + '/' + service + '/' + 'aws4_request'
    string_to_sign = algorithm + '\n' +  amzdate + '\n' +  credential_scope + '\n' +  hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
    signing_key = getSignatureKey(secret_key, datestamp, region, service)
    signature = hmac.new(signing_key, string_to_sign.encode('utf-8'), hashlib.sha256).hexdigest()
    authorization_header = algorithm + ' ' + 'Credential=' + access_key + '/' + credential_scope + ', ' \
        + 'SignedHeaders=' + signed_headers + ', ' + 'Signature=' + signature
    
    request_url = 'https://%s%s' % (host, canonical_uri)
    headers['Authorization'] = authorization_header
    if content_type is not None:
        headers['content-type'] = content_type
    return request_url, headers


def terminal_calendar(events, cols=3):
    """ Get calendar covering all dates, with provided dates colored and labeled """
    if len(events.keys()) == 0:
        return ''
    # events is {'date': 'label'}
    _dates = sorted(events.keys())
    _labels = set(events.values())
    labels = dict(zip(_labels, [str(41 + i) for i in range(0, len(_labels))]))

    start_year = _dates[0].year
    end_year = _dates[-1].year

    # start and end rows
    row1 = int((_dates[0].month - 1) / cols)
    row2 = int((_dates[-1].month - 1) / cols) + 1

    # generate base calendar array
    Calendar = calendar.Calendar()
    cal = []
    for yr in range(start_year, end_year+1):
        ycal = Calendar.yeardatescalendar(yr, width=cols)
        if yr == start_year and yr == end_year:
            ycal = ycal[row1:row2]
        elif yr == start_year:
            ycal = ycal[row1:]
        elif yr == end_year:
            ycal = ycal[:row2]
        cal.append(ycal)

    # month and day headers
    months = calendar.month_name
    days = 'Mo Tu We Th Fr Sa Su'
    hformat = '{:^20}  {:^20}  {:^20}\n'
    rformat = ' '.join(['{:>2}'] * 7) + '  '

    # create output
    col0 = '\033['
    col_end = '\033[0m'
    out = ''
    for iy, yrcal in enumerate(cal):
        out += '{:^64}\n\n'.format(_dates[0].year + iy)
        for mrow in yrcal:
            mnum = mrow[0][2][3].month
            names = [months[mnum], months[mnum+1], months[mnum+2]]
            out += hformat.format(names[0], names[1], names[2])
            out += hformat.format(days, days, days)
            for r in range(0, len(mrow[0])):
                for c in range(0, cols):
                    if len(mrow[c]) == 4:
                        mrow[c].append([''] * 7)
                    if len(mrow[c]) == 5:
                        mrow[c].append([''] * 7)
                    wk = []
                    for d in mrow[c][r]:
                        if d == '' or d.month != (mnum + c):
                            wk.append('')
                        else:
                            string = str(d.day).rjust(2, ' ')
                            if d in _dates:
                                string = '%s%sm%s%s' % (col0, labels[events[d]], string, col_end)
                            wk.append(string)
                    out += rformat.format(*wk)
                out += '\n'
            out += '\n'
    # print labels
    for lbl, col in labels.items():
        vals = list(_labels)
        out += '%s%sm%s (%s)%s\n' % (col0, col, lbl, vals.count(lbl), col_end)
    out += '%s total dates' % len(_dates)
    return out



import os
import unittest

#import satsearch.config as config

#from satsearch.scene import Scenes
from datetime import datetime
from satstac import utils


class Test(unittest.TestCase):

    path = os.path.dirname(__file__)

    remote_url = 'https://landsat-stac.s3.amazonaws.com/catalog.json'

    def test_dict_merge(self):
        dict1 = {
            'key1': {
                'subkey1': 'val'
            }
        }
        dict2 = {
            'key1': {
                'subkey2': 'val'
            },
            'key2': 'val'
        }
        _dict = utils.dict_merge(dict1, dict2)
        assert('key1' in _dict)
        assert('key2' in _dict)
        assert('subkey1' in _dict['key1'])
        assert('subkey2' in _dict['key1'])
        _dict = utils.dict_merge(dict1, dict2, add_keys=False)
        assert('key1' in _dict)
        assert('key2' not in _dict)
        assert('subkey1' in _dict['key1'])
        assert('subkey2' not in _dict['key1'])

    def test_download_nosuchfile(self):
        with self.assertRaises(Exception):
            utils.download_file('http://nosuchfile')

    def test_get_s3_signed_url(self):
        url = utils.get_s3_signed_url(self.remote_url)
        assert(len(url) == 2)

    def test_get_s3_public_url(self):
        envs = dict(os.environ)
        if 'AWS_ACCESS_KEY_ID' in envs:
            del os.environ['AWS_ACCESS_KEY_ID']
        if 'AWS_BUCKET_ACCESS_KEY_ID' in envs:
            del os.environ['AWS_BUCKET_ACCESS_KEY_ID']        
        url = utils.get_s3_signed_url(self.remote_url)
        assert(len(url) == 2)
        assert(url[0] == self.remote_url)
        assert(url[1] is None)
        os.environ.clear()
        os.environ.update(envs)

    def test_terminal_calendar(self):
        """ Get calendar """
        events = {
            datetime(2018,1,1).date(): "event1",
            datetime(2018,4,25).date(): "event2"
        }
        cal = utils.terminal_calendar(events)
        self.assertEqual(len(cal), 1136)
        self.assertTrue(' 2018 ' in cal)
        self.assertTrue(' January ' in cal)
        self.assertTrue(' March ' in cal)

    def test_empty_terminal_calendar(self):
        cal = utils.terminal_calendar({})
        print(cal)


import json
import os

from .version import __version__
from .thing import Thing, STACError
from .config import STAC_VERSION


class Catalog(Thing):

    def __init__(self, data, root=None, **kwargs):
        """ Initialize a catalog with a catalog file """
        super(Catalog, self).__init__(data, **kwargs)

    @property
    def stac_version(self):
        """ Get the STAC version of this catalog """
        return self._data['stac_version']

    @property
    def description(self):
        """ Get catalog description """
        return self._data.get('description', '')

    @classmethod
    def create(cls, id='stac-catalog', title='A STAC Catalog', 
               description='A STAC Catalog', root=None, **kwargs):
        """ Create new catalog """
        kwargs.update({
            'id': id,
            'stac_version': STAC_VERSION,
            'title': title,
            'description': description,
            'links': []
        })
        return cls(kwargs, root=root)

    def children(self):
        """ Get child links """
        # TODO = should this be tested if Collection and return mix of Catalogs and Collections?
        for l in self.links('child'):
            yield Catalog.open(l)

    def catalogs(self):
        """ Recursive get all catalogs within this Catalog """
        for cat in self.children():
            for subcat in cat.children():
                yield subcat
                yield from subcat.catalogs()
            yield cat

    def collections(self):
        """ Recursively get all collections within this Catalog """
        for cat in self.children():
            if 'extent' in cat._data.keys():
                yield Collection.open(cat.filename)
                # TODO - keep going? if other Collections can appear below a Collection
            else:
                yield from cat.collections()

    def items(self):
        """ Recursively get all items within this Catalog """
        for item in self.links('item'):
            yield Item.open(item)
        for child in self.children():
            yield from child.items()

    def add_catalog(self, catalog, basename='catalog'):
        """ Add a catalog to this catalog """
        if self.filename is None:
            raise STACError('Save catalog before adding sub-catalogs')
        # add new catalog child link
        child_link = '%s/%s.json' % (catalog.id, basename)
        child_fname = os.path.join(self.path, child_link)
        child_path = os.path.dirname(child_fname)
        root_links = self.links('root')
        root_link = root_links[0] if len(root_links) > 0 else self.filename
        root_path = os.path.dirname(root_link)
        self.add_link('child', child_link)
        self.save()
        # strip self, parent, child links from catalog and add new links
        catalog.clean_hierarchy()
        catalog.add_link('root', os.path.relpath(root_link, child_path))
        catalog.add_link('parent', os.path.relpath(self.filename, child_path))
        # create catalog file
        catalog.save(filename=child_fname)
        return self

    def add_collection(self, catalog, basename='collection'):
        """ Add a collection to this catalog """
        return self.add_catalog(catalog, basename=basename)


# import and end of module prevents problems with circular dependencies.
# Catalogs use Items and Items use Collections (which are Catalogs)
from .item import Item
from .collection import Collection



import os
import unittest

from satstac import ItemCollection, Item
from shutil import rmtree

testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    path = os.path.join(testpath, 'test-item')

    @classmethod
    def tearDownClass(cls):
        """ Remove test files """
        if os.path.exists(cls.path):
            rmtree(cls.path)

    def load_items(self):
        return ItemCollection.load(os.path.join(testpath, 'items.json'))

    def test_load(self):
        """ Initialize Scenes with list of Scene objects """
        items = self.load_items()
        assert(len(items._collections) == 1)
        assert(len(items) == 2)
        assert(isinstance(items[0], Item))

    def test_save(self):
        """ Save items list """
        items = self.load_items()
        fname = os.path.join(testpath, 'save-test.json')
        items.save(fname)
        assert(os.path.exists(fname))
        os.remove(fname)
        assert(not os.path.exists(fname))

    def test_collection(self):
        """ Get a collection """
        items = self.load_items()
        col = items.collection('landsat-8-l1')
        assert(col.id == 'landsat-8-l1')

    def test_no_collection(self):
        """ Attempt to get non-existent collection """
        items = self.load_items()
        col = items.collection('nosuchcollection')
        assert(col is None)

    def test_bbox(self):
        items = self.load_items()
        bbox = items.bbox()
        assert(len(bbox) == 4)
        assert(bbox == [11.984710693359375, 44.815941348210835, 12.752380371093748, 45.67740123855739])

    def test_center(self):
        items = self.load_items()
        center = items.center()
        assert(center == [45.24667129338411, 12.368545532226562])

    def test_get_properties(self):
        """ Get set of properties """
        items = self.load_items()
        p = items.properties('eo:platform')
        assert(len(p) == 1)
        assert(p[0] == 'landsat-8')

    def test_print_items(self):
        """ Print summary of items """
        items = self.load_items()
        print(items.summary())

    def test_dates(self):
        """ Get dates of all items """
        items = self.load_items()
        dates = items.dates()
        assert(len(dates) == 1)

    def test_text_calendar(self):
        """ Get calendar """
        items = self.load_items()
        cal = items.calendar()
        assert(len(cal) > 250)

    def test_download_thumbnails(self):
        """ Download all thumbnails """
        items = self.load_items()
        fnames = items.download(key='thumbnail')
        for f in fnames:
            assert(os.path.exists(f))
            os.remove(f)
            assert(not os.path.exists(f))
        #shutil.rmtree(os.path.join(testpath, 'landsat-8-l1'))

    def test_filter(self):
        items = self.load_items()
        items.filter('eo:cloud_cover', [100])
        assert(len(items) == 1)

    def test_download_assets(self):
        """ Download multiple assets from all items """
        items = self.load_items()
        filenames = items.download_assets(keys=['MTL', 'ANG'], path=self.path)
        assert(len(filenames) == 2)
        for fnames in filenames:
            assert(len(fnames) == 2)
            for f in fnames:
                assert(os.path.exists(f))

    def test_download(self):
        """ Download a data file from all items """
        items = self.load_items()
        
        fnames = items.download(key='MTL', path=self.path)
        assert(len(fnames) == 2)
        for f in fnames:
            assert(os.path.exists(f))



import datetime
import json
import os
import shutil
import unittest

from satstac import Item


testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    path = os.path.join(testpath, 'test-item')
    filename = os.path.join(testpath, 'catalog/eo/landsat-8-l1/item.json')

    @classmethod
    def tearDownClass(cls):
        """ Remove test files """
        if os.path.exists(cls.path):
            shutil.rmtree(cls.path)

    #@classmethod
    #def setUpClass(cls):
    #    """ Configure testing class """
    #    config.DATADIR = os.path.join(testpath, config.DATADIR)

    def test_init_without_collection(self):
        with open(self.filename) as f:
            data = json.loads(f.read())
        with open(self.filename.replace('item.json', 'catalog.json')) as f:
            coldata = json.loads(f.read())
        item = Item(data)
        assert(item.id == data['id'])
        assert(item.collection() is None)
        assert(item.eobands == [])
        # now put collection properties here
        data['properties'].update(coldata['properties'])
        item = Item(data)
        assert(len(item.eobands) == 11)
    
    def test_open(self):
        """ Initialize an item """
        item = Item.open(self.filename)
        dt, tm = item.properties['datetime'].split('T')
        assert(str(item.date) == dt)
        assert(item.id == item._data['id'])
        assert(item.geometry == item._data['geometry'])
        assert(str(item) == item._data['id'])
        assert(len(item.bbox) == 4)
        #assert(list(item.keys()) == ['id', 'collection', 'datetime', 'eo:platform'])

    def test_open_with_collection(self):
        item = Item.open(self.filename)
        assert(item.collection().id == 'landsat-8-l1')
        assert(len(item['eo:bands']) == 11)
        assert(item['eo:off_nadir'] == 0)

    def test_class_properties(self):
        """ Test the property functions of the Item class """
        item = Item.open(self.filename)
        l = os.path.join(os.path.dirname(item.filename), item._data['links'][0]['href'])
        assert(os.path.abspath(item.links()[0]) == os.path.abspath(l))

    def test_assets(self):
        """ Get assets for download """
        item = Item.open(self.filename)
        href = item._data['assets']['B1']['href']
        assert(item.assets['B1']['href'] == href)
        assert(item.asset('B1')['href'] == href)
        assert(item.asset('coastal')['href'] == href)

    def test_no_asset(self):
        item = Item.open(self.filename)
        assert(item.asset('no-such-asset') == None)

    def test_substitute(self):
        """ Test string substitution with item fields """
        item = Item.open(self.filename)
        st = item.substitute('${collection}/${date}')
        assert(st == 'landsat-8-l1/2018-10-12')
        st = item.substitute('nosub')
        assert(st == 'nosub')

    def test_download_thumbnail(self):
        """ Get thumbnail for item """
        item = Item.open(self.filename)
        fname = item.download(key='thumbnail', path=self.path)
        assert(os.path.exists(fname))

    def test_download(self):
        """ Retrieve a data file """
        item = Item.open(self.filename)
        fname = item.download(key='MTL', path=self.path)
        assert(os.path.exists(fname))
        fname = item.download(key='MTL', path=self.path)
        assert(os.path.exists(fname))

    def test_download_assets(self):
        """ Retrieve multiple data files """
        item = Item.open(self.filename)
        fnames = item.download_assets(keys=['MTL', 'ANG'], path=self.path)
        for f in fnames:
            assert(os.path.exists(f))

    def test_download_nonexist(self):
        """ Test downloading of non-existent file """
        item = Item.open(self.filename)
        fname = item.download(key='fake_asset', path=self.path)
        assert(fname is None)

    def _test_download_paths(self):
        """ Testing of download paths and filenames """
        item = Item.open(self.filename)
        datadir = config.DATADIR
        filename = config.FILENAME
        config.DATADIR = os.path.join(testpath, '${date}')
        config.FILENAME = '${date}_${id}'
        fname = scene.download('MTL')
        _fname = os.path.join(testpath, '2017-01-01/2017-01-01_testscene_MTL.txt')
        assert(fname == _fname)
        assert(os.path.exists(fname))
        config.DATADIR = datadir
        config.FILENAME = filename
        shutil.rmtree(os.path.join(testpath, '2017-01-01'))
        assert(os.path.exists(fname) == False)

    def _test_create_derived(self):
        """ Create single derived scene """
        scenes = [self.get_test_scene(), self.get_test_scene()]
        scene = Item.create_derived(scenes)
        assert(scene.date == scenes[0].date)
        assert(scene['c:id'] == scenes[0]['c:id'])



import json
import logging
import os
import traceback

from string import Formatter, Template
from datetime import datetime
from dateutil.parser import parse as dateparse

from satstac import __version__, STACError, Thing, utils


logger = logging.getLogger(__name__)


class Item(Thing):

    def __init__(self, *args, **kwargs):
        """ Initialize a scene object """
        super(Item, self).__init__(*args, **kwargs)
        # dictionary of assets by eo:band common_name
        self._assets_by_common_name = None
        # collection instance
        self._collection = kwargs.pop('collection', None)
        # TODO = allow passing in of collection (needed for FC catalogs)

    def collection(self):
        """ Get Collection info for this item """
        if self._collection is None:
            if self.filename is None:
                # TODO - raise exception ?
                return None
            link = self.links('collection')
            if len(link) == 1:
                self._collection = Collection.open(link[0])
        return self._collection

    @property
    def eobands(self):
        """ Get eo:bands from Item or from Collection """
        if 'eo:bands' in self.properties:
            return self.properties['eo:bands']
        elif self.collection() is not None and 'eo:bands' in self.collection().properties:
                return self.collection()['eo:bands']
        return []

    @property
    def properties(self):
        """ Get dictionary of properties """
        return self._data.get('properties', {})

    def __getitem__(self, key):
        """ Get key from properties """
        val = super(Item, self).__getitem__(key)
        if val is None:
            if self.collection() is not None:
                # load properties from Collection
                val = self._collection[key]
        return val

    @property
    def date(self):
        return self.datetime.date()

    @property
    def datetime(self):
        return dateparse(self['datetime'])

    @property
    def geometry(self):
        return self._data['geometry']

    @property
    def bbox(self):
        """ Get bounding box of scene """
        return self._data['bbox']

    @property
    def assets(self):
        """ Return dictionary of assets """
        return self._data.get('assets', {})

    @property
    def assets_by_common_name(self):
        """ Get assets by common band name (only works for assets containing 1 band """
        if self._assets_by_common_name is None and len(self.eobands) > 0:
            self._assets_by_common_name = {}
            for a in self.assets:
                bands = self.assets[a].get('eo:bands', [])
                if len(bands) == 1:
                    eo_band = self.eobands[bands[0]].get('common_name')
                    if eo_band:
                        self._assets_by_common_name[eo_band] = self.assets[a]
        return self._assets_by_common_name

    def asset(self, key):
        """ Get asset for this key OR common_name """
        if key in self.assets:
            return self.assets[key]
        elif key in self.assets_by_common_name:
            return self.assets_by_common_name[key]
        logging.warning('No such asset (%s)' % key)
        return None

    def get_filename(self, path='', filename='${id}', extension='.json'):
        """ Get complete path with filename to this item """
        return os.path.join(
            self.substitute(path),
            self.substitute(filename) + extension
        )

    def substitute(self, string):
        """ Substitute envvars in string with Item values """
        string = string.replace(':', '_colon_')
        subs = {}
        for key in [i[1] for i in Formatter().parse(string.rstrip('/')) if i[1] is not None]:
            if key == 'collection':
                subs[key] = self._data['collection']
            elif key == 'id':
                subs[key] = self.id
            elif key in ['date', 'year', 'month', 'day']:
                vals = {'date': self.date, 'year': self.date.year, 'month': self.date.month, 'day': self.date.day}
                subs[key] = vals[key]
            else:
                subs[key] = self[key.replace('_colon_', ':')]
        return Template(string).substitute(**subs)   

    def download_assets(self, keys=None, **kwargs):
        """ Download multiple assets """
        if keys is None:
            keys = self._data['assets'].keys()
        filenames = []
        for key in keys:
            filenames.append(self.download(key, **kwargs))
        return filenames

    def download(self, key, overwrite=False, path='', filename='${id}', requestor_pays=False):
        """ Download this key (e.g., a band, or metadata file) from the scene """
        asset = self.asset(key)
        if asset is None:
            return None

        _path = self.substitute(path)
        utils.mkdirp(_path)
        _filename = None
        try:
            fname = self.substitute(filename)
            ext = os.path.splitext(asset['href'])[1]
            fout = os.path.join(_path, fname + '_' + key + ext)
            if not os.path.exists(fout) or overwrite:
                _filename = utils.download_file(asset['href'], filename=fout, requestor_pays=requestor_pays)
            else:
                _filename = fout
        except Exception as e:
            _filename = None
            logger.error('Unable to download %s: %s' % (asset['href'], str(e)))
            logger.debug(traceback.format_exc())
        return _filename

    '''
    @classmethod
    def create_derived(cls, scenes):
        """ Create metadata for dervied scene from multiple input scenes """
        # data provenance, iterate through links
        links = []
        for i, scene in enumerate(scenes):
            links.append({
                'rel': 'derived_from',
                'href': scene.links['self']['href']
            })
        # calculate composite geometry and bbox
        geom = scenes[0].geometry
        # properties
        props = {
            'id': '%s_%s' % (scenes[0].date, scenes[0]['eo:platform']),
            'datetime': scenes[0]['datetime']
        }
        collections = [s['c:id'] for s in scenes if s['c:id'] is not None]
        if len(collections) == 1:
            props['c:id'] = collections[0]
        item = {
            'properties': props,
            'geometry': geom,
            'links': links,
            'assets': {}
        }
        return Item(item)        
    '''

# import and end of module prevents problems with circular dependencies.
# Catalogs use Items and Items use Collections (which are Catalogs)
from .collection import Collection


import json
import os
import unittest
import shutil

from satstac import __version__, STACError, Catalog, Collection, Item


testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    path = os.path.join(testpath, 'test-collection')

    @classmethod
    def tearDownClass(cls):
        """ Remove test files """
        if os.path.exists(cls.path):
            shutil.rmtree(cls.path)

    def open_collection(self):
        filename = os.path.join(testpath, 'catalog/eo/landsat-8-l1/catalog.json')
        return Collection.open(filename)

    def test_open(self):
        cat = self.open_collection()
        assert(cat.id == 'landsat-8-l1')

    def test_title(self):
        cat = self.open_collection()
        assert(cat.title == 'Landsat 8 L1')
        keys = cat.keywords
        assert(len(keys) == 1)
        assert(keys[0] == 'landsat')
        version = cat.version
        assert(version == '0.1.0')
        license = cat.license
        assert(license == 'PDDL-1.0')
        assert(len(cat.providers) == 4)
        ext = cat.extent
        assert('spatial' in ext)
        assert('temporal' in ext)
        assert(len(cat.properties))

    def test_add_item(self):
        cat = Catalog.create(root='http://my.cat').save(os.path.join(self.path, 'catalog.json'))
        col = Collection.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/catalog.json'))
        cat.add_catalog(col)
        item = Item.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/item.json'))
        col.add_item(item)
        assert(item.parent().id == 'landsat-8-l1')

    def test_add_item_without_saving(self):
        col = Collection.create()
        item = Item.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/item.json'))
        with self.assertRaises(STACError):
            col.add_item(item)

    def test_add_item_with_subcatalogs(self):
        cat = Catalog.create(root='http://my.cat').save(os.path.join(self.path, 'test_subcatalogs.json'))
        col = Collection.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/catalog.json'))
        cat.add_catalog(col)
        item = Item.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/item.json'))
        col.add_item(item, path='${landsat:path}/${landsat:row}/${date}')
        assert(item.root().id == cat.id)
        assert(item.collection().id == col.id)
        # test code using existing catalogs
        col.add_item(item, '${landsat:path}/${landsat:row}/${date}')
        assert(item.root().id == cat.id)


import os
import unittest

testpath = os.path.dirname(__file__)


class _Test(unittest.TestCase):
    """ Test main module """

    def _test_main(self):
        """ Run main function """
        pass



import os
import sys
import unittest

from shlex import split
from shutil import rmtree
from datetime import datetime as dt

from satstac.cli import parse_args, cli
from satstac import Catalog

testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    def test_parse_no_args(self):
        with self.assertRaises(SystemExit):
            parse_args([''])
        with self.assertRaises(SystemExit):
            parse_args(['-h'])

    def test_parse_args(self):
        input = "create testid 'this is a description'"
        args = parse_args(split(input))
        assert(len(args) == 6)
        assert(args['id'] == 'testid')

    def test_cli_create(self):
        input = "sat-stac create cat 'this is a description'"
        sys.argv = split(input)
        cli()
        assert(os.path.exists('catalog.json'))
        input = "sat-stac create subcat 'this is another description' --root catalog.json"
        sys.argv = split(input)
        cli()
        assert(os.path.exists('subcat/catalog.json'))
        rmtree('subcat')
        os.remove('catalog.json')



import json
import os
import unittest
import shutil

from satstac import __version__, Catalog, STACError, Item


testpath = os.path.dirname(__file__)


class Test(unittest.TestCase):

    path = os.path.join(testpath, 'test-catalog')

    @classmethod
    def tearDownClass(cls):
        """ Remove test files """
        if os.path.exists(cls.path):
            shutil.rmtree(cls.path)

    @classmethod
    def get_catalog(cls):
        """ Open existing test catalog """
        return Catalog.open(os.path.join(testpath, 'catalog/catalog.json'))

    @classmethod
    def create_catalog(cls, name):
        path = os.path.join(cls.path, name)
        return Catalog.create(path)

    def test_init(self):
        with open(os.path.join(testpath, 'catalog/catalog.json')) as f:
            data = json.loads(f.read())
        cat = Catalog(data)
        assert(cat.id == 'stac')

    def test_open(self):
        """ Initialize Catalog with a file """
        cat = self.get_catalog()
        assert(len(cat._data.keys()) == 4)
        assert(cat.id == 'stac')
        assert(len(cat.links())==3)

    def test_properties(self):
        cat = self.get_catalog()
        assert(cat.stac_version == '0.6.0')
        assert(cat.description == 'A STAC of public datasets')

    def test_create(self):
        """ Create new catalog file """
        cat = Catalog.create()
        assert(cat.id == 'stac-catalog')

    def test_create_with_keywords(self):
        path = os.path.join(testpath, 'test-catalog', 'create_with_keywords')
        desc = 'this is a catalog'
        cat = Catalog.create(path, description=desc)
        assert(cat.description == desc)

    def test_links(self):
        root = self.get_catalog()
        child = [c for c in root.children()][0]
        assert(child.parent().id == root.id)

    def test_get_catalogs(self):
        catalogs = [i for i in self.get_catalog().catalogs()]
        assert(len(catalogs) == 4)

    def test_get_collections(self):
        collections = [i for i in self.get_catalog().collections()]
        assert(len(collections) == 2)
        assert(collections[0].id in ['landsat-8-l1', 'sentinel-2-l1c'])
        assert(collections[1].id in ['landsat-8-l1', 'sentinel-2-l1c'])

    def test_get_items(self):
        items = [i for i in self.get_catalog().items()]
        assert(len(items) == 2)

    def test_add_catalog(self):
        cat = Catalog.create(root='http://my.cat').save(os.path.join(self.path, 'catalog.json'))
        col = Catalog.open(os.path.join(testpath, 'catalog/eo/landsat-8-l1/catalog.json'))
        cat.add_catalog(col)
        child = [c for c in cat.children()][0]
        assert(child.id == col.id)

    def test_add_catalog_without_saving(self):
        cat = Catalog.create()
        with self.assertRaises(STACError):
           cat.add_catalog({})



import json
import os
import requests

from logging import getLogger
from urllib.parse import urljoin
from .version import __version__
from .utils import mkdirp, get_s3_signed_url


logger = getLogger(__name__)


class STACError(Exception):
    pass


class Thing(object):

    def __init__(self, data, filename=None):
        """ Initialize a new class with a dictionary """
        self.filename = filename
        self._data = data
        if 'id' not in data:
            raise STACError('ID is required')
        if 'links' not in self._data.keys():
            self._data['links'] = []

    def __repr__(self):
        return self.id

    @classmethod
    def open_remote(self, url, headers={}):
        """ Open remote file """
        resp = requests.get(url, headers=headers)
        if resp.status_code == 200:
            dat = resp.text
        else:
            raise STACError('Unable to open %s' % url)
        return json.loads(dat)

    @classmethod
    def open(cls, filename):
        """ Open an existing JSON data file """
        logger.debug('Opening %s' % filename)
        if filename[0:5] == 'https':
            try:
                dat = cls.open_remote(filename)
            except STACError as err:
                # try signed URL
                url, headers = get_s3_signed_url(filename)
                dat = cls.open_remote(url, headers)
        else:
            if os.path.exists(filename):
                dat = open(filename).read()
                dat = json.loads(dat)
            else:
                raise STACError('%s does not exist locally' % filename)
        return cls(dat, filename=filename)

    def __getitem__(self, key):
        """ Get key from properties """
        props = self._data.get('properties', {})
        return props.get(key, None)

    @property
    def id(self):
        """ Return id of this entity """
        return self._data['id']

    @property
    def path(self):
        """ Return path to this catalog file (None if no filename set) """
        return os.path.dirname(self.filename) if self.filename else None

    def links(self, rel=None):
        """ Get links for specific rel type """
        links = self._data.get('links', [])
        if rel is not None:
            links = [l for l in links if l.get('rel') == rel]
        links = [l['href'] for l in links]
        if self.filename is not None:
            _links = []
            for l in links:
                if os.path.isabs(l) or l[0:4] == 'http':
                    # if absolute or https 
                    link = l
                else:
                    # relative path
                    if self.filename[0:4] == 'http':
                        link = urljoin(os.path.dirname(self.filename) + '/', l)
                    else: 
                        link = os.path.abspath(os.path.join(os.path.dirname(self.filename), l))
                _links.append(link)
            links = _links
        return links

    def root(self):
        """ Get root link """
        links = self.links('root')
        if len(links) == 1:
            return self.open(links[0])
        elif len(links) == 0:
            # i'm the root of myself
            return self
        else:
            raise STACError('More than one root provided')

    def parent(self):
        """ Get parent link """
        links = self.links('parent')
        if len(links) == 1:
            return self.open(links[0])
        elif len(links) == 0:
            return None
        else:
            raise STACError('More than one parent provided')

    def add_link(self, rel, link, type=None, title=None):
        """ Add a new link """
        # if this link already exists do not add it
        for l in self._data['links']:
            if l['rel'] == rel and l['href'] == link:
                return
        l = {'rel': rel, 'href': link}
        if type is not None:
            l['type'] = type
        if title is not None:
            l['title'] = title
        self._data['links'].append(l)


    def clean_hierarchy(self):
        """ Clean links of self, parent, and child links (for moving and publishing) """
        rels = ['self', 'root', 'parent', 'child', 'collection', 'item']
        links = []
        for l in self._data['links']:
            if l['rel'] not in rels:
                links.append(l)
        self._data['links'] = links

    def save(self, filename=None):
        """ Write a catalog file """
        if filename is not None:
            self.filename = filename
        if self.filename is None:
            raise STACError('No filename provided, specify with filename keyword')
        logger.debug('Saving %s as %s' % (self.id, self.filename))
        fname = self.filename
        if self.filename[0:5] == 'https':
            # use signed URL
            signed_url, signed_headers = get_s3_signed_url(self.filename, rtype='PUT',
                                                           public=True, content_type='application/json')
            resp = requests.put(signed_url, data=json.dumps(self._data), headers=signed_headers)
            if resp.status_code != 200:
                raise STACError('Unable to save file to %s: %s' % (self.filename, resp.text))
        else:
            # local file save
            mkdirp(os.path.dirname(fname))
            with open(fname, 'w') as f:
                f.write(json.dumps(self._data))
        return self


__path__ = __import__('pkgutil').extend_path(__path__, __name__)
from .version import __version__
from .thing import Thing, STACError
from .catalog import Catalog
from .collection import Collection
from .item import Item
from .itemcollection import ItemCollection



import os
import sys
import argparse
import logging

from .catalog import Catalog
from .version import __version__


logger = logging.getLogger(__name__)


def parse_args(args):
    desc = 'stac (v%s)' % __version__
    dhf = argparse.ArgumentDefaultsHelpFormatter
    parser0 = argparse.ArgumentParser(description=desc)

    pparser = argparse.ArgumentParser(add_help=False)
    pparser.add_argument('--version', help='Print version and exit', action='version', version=__version__)
    pparser.add_argument('--log', default=2, type=int,
                         help='0:all, 1:debug, 2:info, 3:warning, 4:error, 5:critical')

    # add subcommands
    subparsers = parser0.add_subparsers(dest='command')

    # command 1
    parser = subparsers.add_parser('create', parents=[pparser], help='Create a root catalog', formatter_class=dhf)
    parser.add_argument('id', help='ID of the new catalog')
    parser.add_argument('description', help='Description of new catalog')
    parser.add_argument('--filename', help='Filename of catalog', default='catalog.json')
    parser.add_argument('--root', help='Filename to existing root catalog', default=None)

    # turn Namespace into dictinary
    parsed_args = vars(parser0.parse_args(args))

    return parsed_args


def cli():
    args = parse_args(sys.argv[1:])
    logger.setLevel(args.pop('log') * 10)
    cmd = args.pop('command')

    if cmd == 'create':
        if args['root'] is not None:
            root = Catalog.open(args['root'])
            cat = Catalog.create(id=args['id'], description=args['description'])
            root.add_catalog(cat)
        else:
            cat = Catalog.create(id=args['id'], description=args['description'])
            cat.save(filename=args['filename'])


if __name__ == "__main__":
    cli()



#!/usr/bin/env python
from setuptools import setup, find_packages
from imp import load_source
from os import path
import io

__version__ = load_source('satstac.version', 'satstac/version.py').__version__

here = path.abspath(path.dirname(__file__))

# get the dependencies and installs
with io.open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if 'git+' not in x]

setup(
    name='sat-stac',
    author='Matthew Hanson (matthewhanson)',
    author_email='matt.a.hanson@gmail.com',
    version=__version__,
    description='A Python library for working with Spatio-Temporal Asset Catalogs (STAC)',
    url='https://github.com/sat-utils/sat-stac.git',
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='',
    entry_points={
        'console_scripts': ['sat-stac=satstac.cli:cli'],
    },
    packages=find_packages(exclude=['docs', 'tests*']),
    include_package_data=True,
    install_requires=install_requires,
    dependency_links=dependency_links,
)



import json
import os.path as op
import requests

from logging import getLogger
from .collection import Collection
from .item import Item
from .thing import STACError
from .utils import terminal_calendar, get_s3_signed_url

logger = getLogger(__name__)


class ItemCollection(object):
    """ A GeoJSON FeatureCollection of STAC Items with associated Collections """

    def __init__(self, items, collections=[], search={}):
        """ Initialize with a list of Item objects """
        self._collections = collections
        self._items = items
        self._search = search
        # link Items to their Collections
        cols = {c.id: c for c in self._collections}
        for i in self._items:
            # backwards compatible to STAC 0.6.0 where collection is in properties
            col = i._data.get('collection', None)
            if col is not None:
                if col in cols:
                    i._collection = cols[col]

    @classmethod
    def open_remote(self, url, headers={}):
        """ Open remote file """
        resp = requests.get(url, headers=headers)
        if resp.status_code == 200:
            dat = resp.text
        else:
            raise STACError('Unable to open %s' % url)
        return json.loads(dat)

    @classmethod
    def open(cls, filename):
        """ Load an Items class from a GeoJSON FeatureCollection """
        """ Open an existing JSON data file """
        logger.debug('Opening %s' % filename)
        if filename[0:5] == 'https':
            try:
                data = cls.open_remote(filename)
            except STACError as err:
                # try signed URL
                url, headers = get_s3_signed_url(filename)
                data = cls.open_remote(url, headers)
        else:
            if op.exists(filename):
                data = open(filename).read()
                data = json.loads(data)
            else:
                raise STACError('%s does not exist locally' % filename)
        collections = [Collection(col) for col in data['collections']]
        items = [Item(feature) for feature in data['features']]
        return cls(items, collections=collections, search=data.get('search'))

    @classmethod
    def load(cls, *args, **kwargs):
        """ Load an Items class from a GeoJSON FeatureCollection """
        logger.warning("ItemCollection.load() is deprecated, use ItemCollection.open()")
        return cls.open(*args, **kwargs)

    def __len__(self):
        """ Number of scenes """
        return len(self._items)

    def __getitem__(self, index):
        return self._items[index]

    def dates(self):
        """ Get sorted list of dates for all scenes """
        return sorted(list(set([s.date for s in self._items])))

    def collection(self, id):
        """ Get collection records for this list of scenes """
        cols = [c for c in self._collections if c.id == id]
        if len(cols) == 1:
            return cols[0]
        else:
            return None

    def bbox(self):
        """ Get bounding box of search """
        if 'intersects' in self._search.get('parameters', {}):
            coords = self._search['parameters']['intersects']['geometry']['coordinates']
            lats = [c[1] for c in coords[0]]
            lons = [c[0] for c in coords[0]]
            return [min(lons), min(lats), max(lons), max(lats)]
        else:
            return None

    def center(self):
        if 'intersects' in self._search.get('parameters', {}):
            coords = self._search['parameters']['intersects']['geometry']['coordinates']
            lats = [c[1] for c in coords[0]]
            lons = [c[0] for c in coords[0]]
            return [(min(lats) + max(lats))/2.0, (min(lons) + max(lons))/2.0]
        else:
            return None

    def search_geometry(self):
        if 'intersects' in self._search.get('parameters', {}):
            return self._search['parameters']['intersects']
        else:
            return None

    def properties(self, key, date=None):
        """ Set of values for 'key' property in Items, for specific date if provided """
        if date is None:
            return list(set([i[key] for i in self._items]))
        else:
            return list(set([i[key] for i in self._items if i.date == date]))    

    def summary(self, params=[]):
        """ Print summary of all scenes """
        if len(params) == 0:
            params = ['date', 'id']
        txt = 'Items (%s):\n' % len(self._items)
        txt += ''.join(['{:<25} '.format(p) for p in params]) + '\n'
        for s in self._items:
            txt += ''.join(['{:<25} '.format(s.substitute('${%s}' % p)) for p in params]) + '\n'
        return txt

    def calendar(self):
        """ Get calendar for dates """
        date_labels = {}
        for d in self.dates():
            sensors = self.properties('eo:platform', d)
            if len(sensors) > 1:
                date_labels[d] = 'Multiple'
            else:
                date_labels[d] = sensors[0]
        return terminal_calendar(date_labels)

    def save(self, filename):
        """ Save scene metadata """
        with open(filename, 'w') as f:
            f.write(json.dumps(self.geojson()))

    def geojson(self):
        """ Get Items as GeoJSON FeatureCollection """
        features = [s._data for s in self._items]
        geoj = {
            'type': 'FeatureCollection',
            'features': features,
            'collections': [c._data for c in self._collections],
        }
        if self._search is not None:
            geoj['search'] = self._search
        return geoj

    def filter(self, key, values):
        """ Filter scenes on key matching value """
        items = []
        for val in values:
            items += list(filter(lambda x: x[key] == val, self._items))
        self._items = items

    def download_assets(self, *args, **kwargs):
        filenames = []
        for i in self._items:
            fnames = i.download_assets(*args, **kwargs)
            if len(fnames) > 0:
                filenames.append(fnames)
        return filenames

    def download(self, *args, **kwargs):
        """ Download all Items """
        dls = []
        for i in self._items:
            fname = i.download(*args, **kwargs)
            if fname is not None:
                dls.append(fname)
        return dls
